/*
 * Created on 21.10.2008
 */
package Alignment;


import RealValue.Individual;
import Utilities.ClassicEvaluation;
import Utilities.Coords;
import Utilities.EvalParameter;
import Utilities.Graph;

/**
 * @author Thomas Fober
 *         Philipps-Universitt Marburg
 *         Fachbereich Mathematik und Informatik
 *         thomas@mathemetik.uni-marburg.de
 */
public class HungarianAlignment {
	public int[][] alignment;
	public double[][] ungarianMatrix;
	public double fitnessAlignment;
	public Graph[] graphs;
	int maxAlignmentLength;
	
	public HungarianAlignment(){
		
	}
	
	public HungarianAlignment(int[][] star, double fitness, Graph[] graphs, double time, String method, String termination){
		alignment = star;
		fitnessAlignment = fitness;
		this.graphs = graphs;
	}
	
	public HungarianAlignment(double threshold, Individual overlay, double param1, double param2){
		overlay.calcDistances();
		maxAlignmentLength = overlay.molecules[0].size() + overlay.molecules[1].size();
		ungarianMatrix = new double[maxAlignmentLength][maxAlignmentLength];
		for(int i = 0; i < ungarianMatrix.length; i++){
			for(int j = 0; j < ungarianMatrix[0].length; j++){
				if(i < overlay.distances.length && j < overlay.distances[0].length){
					ungarianMatrix[i][j] = overlay.distances[i][j]; //here only distance (VORZEICHENWECHSEL)!!!
					//ungarianMatrix[i][j] = Math.exp(overlay.distances[i][j] * param1); //here distance + node label: distance in range[0,inf[
					//Coords a = (Coords)overlay.molecules[0].get(i);
					//Coords b = (Coords)overlay.molecules[1].get(j);
					//ungarianMatrix[i][j] = param2 * ungarianMatrix[i][j] + (1 - param2) * ((a.getLabel().equalsIgnoreCase(b.getLabel())) ? 1 : 0);
				}else if(i > overlay.distances.length && j < overlay.distances[0].length){
					//ungarianMatrix[i][j] = Math.exp(threshold * param1);
					ungarianMatrix[i][j] = threshold;
				}else if(i < overlay.distances.length && j > overlay.distances[0].length){
					//ungarianMatrix[i][j] = Math.exp(threshold * param1);
					ungarianMatrix[i][j] = threshold;
				}else{
					ungarianMatrix[i][j] = 0;
				}
			}
		}
		//apply hungarian2 algo
		Hungarian2 hungarian = new Hungarian2();
		HungarianProblem problem = new HungarianProblem();
		hungarian.init(problem, ungarianMatrix, true);
		hungarian.solve(problem);
		alignment = problem.zuordnung;
		int remove = 0;
		for(int i = 0; i < alignment[0].length; i++){
			if(alignment[0][i] >= graphs[0].numberOfNodes){
				alignment[0][i] = -1;
			}
			if(alignment[1][i] >= graphs[1].numberOfNodes){
				alignment[1][i] = -1;
			}
			if(alignment[0][i] == -1 && alignment[1][i] == -1){
				remove++;
			}
		}
		int[][] tmp = new int[2][alignment[0].length-remove];
		int jumps = 0;
		for(int i = 0; i < alignment[0].length; i++){
			if(alignment[0][i] != -1 || alignment[1][i] != -1){
				tmp[0][i - jumps] = alignment[0][i];
				tmp[1][i - jumps] = alignment[1][i];
			}else{
				jumps++;
			}
		}
		alignment = tmp;
		fitnessAlignment = ClassicEvaluation.evaluate(alignment, graphs, new EvalParameter());
	}
	
	public HungarianAlignment(Graph g1, Graph g2, double threshold, Individual overlay, double param1, double param2){
		graphs = new Graph[2];
		graphs[0] = g1;
		graphs[1] = g2;
		maxAlignmentLength = g1.numberOfNodes + g2.numberOfNodes - 1;
		overlay.calcDistances();
		ungarianMatrix = new double[maxAlignmentLength][maxAlignmentLength];
		for(int i = 0; i < ungarianMatrix.length; i++){
			for(int j = 0; j < ungarianMatrix[0].length; j++){
				if(i < overlay.distances.length && j < overlay.distances[0].length){
					//ungarianMatrix[i][j] = overlay.distances[i][j]; //here only distance VORZEICHENWECHSEL!!!
					ungarianMatrix[i][j] = Math.exp(overlay.distances[i][j] * param1); //here distance + node label: distance in range[0,inf[
					Coords a = (Coords)overlay.molecules[0].get(i);
					Coords b = (Coords)overlay.molecules[1].get(j);
					ungarianMatrix[i][j] = param2 * ungarianMatrix[i][j] + (1 - param2) * ((a.getLabel().equalsIgnoreCase(b.getLabel())) ? 1 : 0);
				}else if(i > overlay.distances.length && j < overlay.distances[0].length){
					ungarianMatrix[i][j] = threshold;
				}else if(i < overlay.distances.length && j > overlay.distances[0].length){
					ungarianMatrix[i][j] = threshold;
				}else{
					ungarianMatrix[i][j] = 0;
				}
			}
		}
		//apply hungarian2 algo
		Hungarian2 hungarian = new Hungarian2();
		HungarianProblem problem = new HungarianProblem();
		hungarian.init(problem, ungarianMatrix, false);
		hungarian.solve(problem);
		alignment = problem.zuordnung;
		int remove = 0;
		for(int i = 0; i < alignment[0].length; i++){
			if(alignment[0][i] >= graphs[0].numberOfNodes){
				alignment[0][i] = -1;
			}
			if(alignment[1][i] >= graphs[1].numberOfNodes){
				alignment[1][i] = -1;
			}
			if(alignment[0][i] == -1 && alignment[1][i] == -1){
				remove++;
			}
		}
		int[][] tmp = new int[2][alignment[0].length-remove];
		int jumps = 0;
		for(int i = 0; i < alignment[0].length; i++){
			if(alignment[0][i] != -1 || alignment[1][i] != -1){
				tmp[0][i - jumps] = alignment[0][i];
				tmp[1][i - jumps] = alignment[1][i];
			}else{
				jumps++;
			}
		}
		alignment = tmp;
		fitnessAlignment = ClassicEvaluation.evaluate(alignment, graphs, new EvalParameter());
	}
}
