package Alignment;

import Utilities.*;
import java.io.Serializable;
import java.util.*;

/**
 * @author Thomas Fober
 *         Philipps-Universitt Marburg
 *         Fachbereich Mathematik und Informatik
 *         thomas@mathemetik.uni-marburg.de
 */
public class StarAlign implements Serializable{

	private static final long serialVersionUID = -5999001462796981916L;

	public StarAlign() {
	}
	
	public static HungarianAlignment starAlign(Graph[] graphs, double threshold, EvalParameter parameter, double parameter1, double parameter2) {
		long startTime = System.currentTimeMillis();
		HungarianAlignment[][] pairwise = AllPairwiseAlignments.getAllPairwiseAlignments(graphs, threshold, parameter1, parameter2);
		HungarianAlignment MGA = StarAlign.generateStarAlign(pairwise, graphs, parameter);
		long stopTime = System.currentTimeMillis();
		//if(MGA!=null)MGA.time = stopTime - startTime;
		return MGA;
	}
	
	/**Generates a Multiple Graph Alignment, using a star align merging algorithm
	 * 
	 */
	private static HungarianAlignment generateStarAlign(HungarianAlignment[][] pairwise, Graph[] graphs, EvalParameter parameter){
//		System.out.println("Calculating alignment of "+graphs.length+" graphs using a greedy heuristic...");
		//only 2 graphs, no staralign required
		if (pairwise.length == 2){
			HungarianAlignment res = pairwise[0][1];
//			System.out.println("Calculation done!");
			return res;
		}
		//multiple alignment calculation
		int gr = pairwise.length;								//number of pairwise alignments
		HungarianAlignment best = new HungarianAlignment();							//starting best solution
		best.fitnessAlignment = Double.NEGATIVE_INFINITY;					//fitness initialization
		//try all lines as pivot rows
		for (int pivot = 0; pivot < gr; pivot++){
			Vector<Integer>[] star = new Vector[gr];				//array for storing indices dynamically
			boolean notInit = true;								// true if no entry in star
			//calculate multi alignment over all pairwise alignments
			for (int i = 0; i < gr; i++){
				if (pivot == i) continue;		//same graphs
				HungarianAlignment current = pairwise[pivot][i];
//				make sure, pivot graph is in the first position
				int p = 1;
				int q = 0;
				if (pivot < i) {
					p = 0;
					q = 1;
				}
				Vector<Integer> currentP = new Vector<Integer>();
				Vector<Integer> currentQ = new Vector<Integer>();
				for (int j = 0; j < current.alignment[0].length; j++) {
					currentP.add(current.alignment[p][j]);				//Pivot row for solution
					currentQ.add(current.alignment[q][j]);				//corresponding row in first pairwise alignment
				}
				//set intitial alignment, first 2 rows can be inserted as given by the pairwise solution
				if (notInit){ 
					star[pivot] = currentP;
					star[i] = currentQ;
					notInit=false;
				}
				//add next pairwise to star
				else{
					Vector<Integer> next = new Vector<Integer>();				//next row to be added
					for (int pi = 0; pi < star[pivot].size(); pi++){			//current position
						int pivotEntry = (Integer)star[pivot].get(pi);			// entry in pivot row in result alignment at current position pi
						boolean found = false;									//has corresponding entry
						for (int ni = 0; ni < currentP.size(); ni++){			//current position in pivot graph in next alignment
							int compare = (Integer)currentP.get(ni);			//entry iin pivot row of next alignment at position ni
							
							if (pivotEntry == compare){							//if identical, the corresponding position on new alignment has been found
								int newEntry = (Integer)currentQ.get(ni);		//next entry to be added to solution
								next.add(pi, newEntry);							//add entry to solution
								if (compare == -1) currentP.set(ni, -2);		//if entry in pivot row is dummy
								found = true;									//next entry to be added found
								break;
							}
						}
						if (!found) next.add(pi, -1);							//if pivot row contains additional dummies
					}
					//if new line contains additionsal dummies
					if (currentP.size() > star[pivot].size()){
						for (int ni = 0; ni < currentP.size(); ni++){
							int compare = (Integer)currentP.get(ni);
							if (compare == -1){
								next.add(currentQ.get(ni));
								for (Vector<Integer> arr : star){
									if (arr==null) continue;
									if (arr.size()>0) arr.add(-1);
								}
							}
						}
					}
					star[i] = next;
				}
			}
			Solution sol;
			HungarianAlignment MGAStar = new HungarianAlignment(vectorToSol(star), 0, graphs, 0, "n/A", "n/A");
			double score;
			score = ClassicEvaluation.evaluate(MGAStar.alignment, graphs, parameter);
			MGAStar.fitnessAlignment = Math.round(score * 100) / 100;
			if (score > best.fitnessAlignment){
				best = MGAStar;
			}
		}
//		System.out.println("Calculation done!");
		return best;
	}
	
	
	public static int[][] vectorToSol(Vector<Integer>[] vec) {
		int maxlength = 0;
		for (Vector<Integer> test : vec) {
			if(test.size() > maxlength) maxlength = test.size();
		}
		int[][] sol = new int[vec.length][maxlength];
		for (int i = 0; i < vec.length; i++) {
			for (int j = 0; j < vec[0].size(); j++) {
				sol[i][j] = vec[i].get(j); 
			}
		}
		return sol;
	}
}
