/*
 * Created on 09.08.2007
 *
 */
package Parser;
import java.io.*;
import java.util.Vector;
import Utilities.Coords;
import Utilities.Graph;

/**
 * @author Thomas Fober & Marco Mernberger
 *         Philipps-Universitt Marburg
 *         Fachbereich Mathematik und Informatik
 *         {thomas, mernberger}@mathemetik.uni-marburg.de
 */
public class ChemParser implements FileParser {

	private double[][] distances;
	private String[] nodes;
	private Vector<Coords> coords;
	private int numberOfNodes;
	
	public ChemParser(){
	}
	
	
	/**Creates a graph out of a given file
	 * @param path path of file to parse
	 * @param delta cut-off value for edges
	 */
	public Graph createGraph(String path, double delta){
		this.parse(path);
		this.trimDistance(delta);
		this.createNodes();
		return new Graph(this.numberOfNodes, null, this.distances, this.nodes, this.coords, path);
		
	}
	
	/**Parses a given input file for graph creation.
	 * 
	 */
	private void parse(String path){
			try{
				//open file
				BufferedReader file = new BufferedReader(new FileReader(path));
				file.readLine();
				file.readLine();
				//reading atom data
				String f = file.readLine();
				String[] nums = f.trim().split("\\s+");
				int numberNodes = Integer.parseInt(nums[0]);
				int numberBonds = Integer.parseInt(nums[1]);
				for (int i = 0; i < 6; i++) 
					file.readLine();
				Vector<Coords> coords = new Vector<Coords>();
				Vector<Integer> hAtome = new Vector<Integer>();
				Vector<Integer> map = new Vector<Integer>();
				for (int i = 0; i < numberNodes; i++){
					String next = file.readLine().trim();
					String[] lineC = next.split("\\s+");
					if (lineC[5].equals("H")){
						hAtome.addElement(Integer.parseInt(lineC[0]));
						continue;
					}
					else{
					Coords actual = new Coords(Double.parseDouble(lineC[2]), Double.parseDouble(lineC[3]), Double.parseDouble(lineC[4]), lineC[5]);
					map.addElement(Integer.parseInt(lineC[0]));
					coords.addElement(actual);
					}
				}
				
				//Creating distance matrix
				double[][] distance = new double[coords.size()][coords.size()];
				for (int i = 0; i < coords.size();i++){
					for (int j = 0; j < coords.size();j++){
						if (i==j) distance[i][j] = 0;
						else distance[i][j] = Math.sqrt(Math.pow((coords.get(i).getX()-coords.get(j).getX()),2) + Math.pow((coords.get(i).getY()-coords.get(j).getY()),2) + Math.pow((coords.get(i).getZ()-coords.get(j).getZ()),2));
					}
				}
				//creating bond matrix
				file.readLine();
				int[][] bonds = new int[map.size()][map.size()];
				for (int i=0; i < numberBonds; i++){
					String next = file.readLine().trim();
					String[] lineC = next.split("\\s+");
					int xa = Integer.parseInt(lineC[1]);
					int ya = Integer.parseInt(lineC[2]);
					if (hAtome.contains(xa) || hAtome.contains(ya)) continue;
					else {
						int x = map.indexOf(xa);
						int y = map.indexOf(ya);
						if (lineC[3].equals("ar"))bonds[x][y] = 4;
						else if (lineC[3].equals("am"))bonds[x][y] = 5;
						else bonds[x][y] = Integer.parseInt(lineC[3]);
					}
				}
				file.close();
				this.distances = distance;
				this.coords = coords;
				this.numberOfNodes = coords.size();
				}
			catch (IOException e){
				System.out.println("File not found error");
			}

	}
	
	/**Creates List of nodes labels
	 * 
	 */
	private void createNodes(){
		this.nodes = new String[this.coords.size()];
		for (int i = 0; i < coords.size(); i++){
			this.nodes[i] = coords.get(i).getLabel();
		}
	}
	
	/**Removes edges with distance greater cutoff value
	 * 
	 */
	private void trimDistance(double delta){
		for (int i = 0; i < distances.length; i++){
			for (int j = 0; j < distances[0].length; j++){
				if (distances[i][j]>delta) distances[i][j] = Double.POSITIVE_INFINITY;
			}
		}
	}
	 
}
