/**02.11.2007
 * 15:43:46
 * 2007
 * Description:
 */
package Parser;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.Vector;
import Utilities.Coords;
import Utilities.Graph;

/**
 * @author Thomas Fober & Marco Mernberger
 *         Philipps-Universitt Marburg
 *         Fachbereich Mathematik und Informatik
 *         {thomas, mernberger}@mathemetik.uni-marburg.de
 */
public class RlbCoorParser implements FileParser{

	
	private double[][] distances;
	private String[] nodes;
	private int[][] bonds;
	private String path;
	private Vector<Coords> coords;
	private int numberOfNodes;
	
	
	public RlbCoorParser(){
	}
	
	/**Creates a graph out of a given file
	 * @param path path of file to parse
	 * @param delta cut-off value for edges
	 */
	public Graph createGraph(String path, double delta){
		this.parse(path);
		this.trimDistance(delta);
		this.createNodes();
		return new Graph(this.numberOfNodes, this.bonds, this.distances, this.nodes, this.coords, this.path);
	}
	
	/**Parses a given input file for graph creation.
	 * 
	 */
	private void parse(String path){
			try{
				//open file
				BufferedReader file = new BufferedReader(new FileReader(path));
				//file.readLine();
				
				//construct and search for 'path.list' file

				
				String listFile = path.replace("rlbcoor", "list");
				File list = new File(listFile);
				Vector<Coords> coords;
				if(!list.exists())
				{
					//reading atom data
					//String f = file.readLine();
					//int numberNodes = this.getNumberNodes(path);
					coords = new Vector<Coords>();
					String next = file.readLine();
					while (!next.equalsIgnoreCase("   _relibase_surface_pseudocentre_z")) next = file.readLine();
					while(next != null && !next.equalsIgnoreCase("")){
						//for (int i = 0; i < numberNodes; i++){
						next = file.readLine();
						if (next == null || next.equalsIgnoreCase("")) continue;
						next.trim();
						String[] lineC = next.split("\\s+");
						Coords actual = new Coords(Double.parseDouble(lineC[7]), Double.parseDouble(lineC[8]), Double.parseDouble(lineC[9]), lineC[1]);
						coords.addElement(actual);
					}
				}
				else
				{
					BufferedReader li = new BufferedReader(new FileReader(listFile));
					String t = li.readLine();
					String[] tm = t.split("\\s+");
					Vector<Integer> contained = new Vector<Integer>();
					for(int i = 0; i < tm.length; i++)
					{
						contained.add(Integer.parseInt(tm[i]));
					}
					
					//remove all pseudocenters that are not in contained
					int currentIndex = -1;
					coords = new Vector<Coords>();
					String next = file.readLine();
					while (!next.equalsIgnoreCase("   _relibase_surface_pseudocentre_z")) next = file.readLine();
					while(next != null && !next.equalsIgnoreCase("")){
						currentIndex++;
						if(!contained.contains(currentIndex)) continue;
						//for (int i = 0; i < numberNodes; i++){
						next = file.readLine();
						if (next == null || next.equalsIgnoreCase("")) continue;
						next.trim();
						String[] lineC = next.split("\\s+");
						Coords actual = new Coords(Double.parseDouble(lineC[7]), Double.parseDouble(lineC[8]), Double.parseDouble(lineC[9]), lineC[1]);
						coords.addElement(actual);
					}
				}
				int numberNodes = coords.size();
				//Creating distance matrix
				double[][] distance = new double[coords.size()][coords.size()];
				for (int i = 0; i < coords.size();i++){
					for (int j = 0; j < coords.size();j++){
						if (i==j) distance[i][j] = 0;
						else distance[i][j] = Math.sqrt(Math.pow((coords.get(i).getX()-coords.get(j).getX()),2) + Math.pow((coords.get(i).getY()-coords.get(j).getY()),2) + Math.pow((coords.get(i).getZ()-coords.get(j).getZ()),2));
					}
				}
				//creating bond matrix
				int[][] bonds = null;
				file.close();
				this.bonds = bonds;
				this.distances = distance;
				this.numberOfNodes = numberNodes;
				this.path = path;
				this.coords = coords;
				}
			catch (IOException e){
				System.out.println("File not found error");
			}

	}
	
	/**Creates List of nodes labels
	 * 
	 */
	private void createNodes(){
		this.nodes = new String[this.coords.size()];
		for (int i = 0; i < coords.size(); i++){
			this.nodes[i] = coords.get(i).getLabel();
		}
	}
	
	/**Removes edges with distance greater cutoff value
	 * 
	 */
	private void trimDistance(double delta){
		for (int i = 0; i < distances.length; i++){
			for (int j = 0; j < distances[0].length; j++){
				if (distances[i][j]>delta) distances[i][j] = Double.POSITIVE_INFINITY;
			}
		}
	}
}
